MODULE p4zsink_kriest
   !!======================================================================
   !!                   ***  MODULE p4zsink_kriest  ***
   !! TOP :   PISCES Compute vertical flux of particulate matter due to gravitational sinking
   !!         Kriest parameterization
   !!======================================================================
   !! History :   1.0  !  2004     (O. Aumont) Original code
   !!             2.0  !  2007-12  (C. Ethe, G. Madec)  F90
   !!----------------------------------------------------------------------
#if defined key_pisces  &&  defined key_kriest
   !!----------------------------------------------------------------------
   !!   'key_pisces'    and                                PISCES bio-model
   !!   'key_kriest'                                          kriest option
   !!----------------------------------------------------------------------
   !!   p4z_sink_kriest :  Compute vertical flux of particulate matter due 
   !!                      to gravitational sinking (Kriest parameterization)
   !!----------------------------------------------------------------------
   USE oce_trc         !
   USE trp_trc
   USE sms
   USE p4zsink2
   USE prtctl_trc

   IMPLICIT NONE
   PRIVATE

   PUBLIC   p4z_sink_kriest    ! called in p4zbio.F90

   !!* Substitution
#  include "domzgr_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/TOP 2.0 , LOCEAN-IPSL (2007) 
   !! $Header:$ 
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE p4z_sink_kriest
      !!---------------------------------------------------------------------
      !!                ***  ROUTINE p4z_sink_kriest  ***
      !!
      !! ** Purpose :   Compute vertical flux of particulate matter due to 
      !!              gravitational sinking - Kriest parameterization
      !!
      !! ** Method  : - ???
      !!---------------------------------------------------------------------
      INTEGER  :: ji, jj, jk
      INTEGER  :: iksed
      REAL(wp) :: zagg1, zagg2, zagg3, zagg4, zagg5, zaggsi, zaggsh
      REAL(wp) :: zagg , zaggdoc, znumdoc
      REAL(wp) :: znum , zeps, zfm, zgm, zsm
      REAL(wp) :: zdiv , zdiv1, zdiv2, zdiv3, zdiv4, zdiv5
      REAL(wp) :: zval1, zval2, zval3, zval4
      REAL(wp) :: zstep
#if defined key_trc_dia3d
      REAL(wp) ::   zrfact2
#endif
      REAL(wp), DIMENSION(jpi,jpj,jpk) ::   znum3d
      REAL(wp), DIMENSION(jpi,jpj,jpk) ::   sinking, sinking2
      REAL(wp), DIMENSION(jpi,jpj,jpk) ::   sinkfer
      REAL(wp), DIMENSION(jpi,jpj,jpk) ::   sinkcal, sinksil
      CHARACTER (len=25) :: charout

      !!---------------------------------------------------------------------

       zstep=rfact2/rjjss      ! Time step duration for biology


!     Initialisation of variables used to compute Sinking Speed
!     ---------------------------------------------------------

       znum3d(:,:,:) = 0.e0
       iksed = 10
       zval1 = 1. + xkr_zeta
       zval2 = 1. + xkr_zeta + xkr_eta
       zval3 = 1. + xkr_eta

!     Computation of the vertical sinking speed : Kriest et Evans, 2000
!     -----------------------------------------------------------------
    
      DO jk = 1, jpkm1
         DO jj = 1, jpj
            DO ji = 1, jpi
               IF( tmask(ji,jj,jk) /= 0.e0 ) THEN
                  znum = trn(ji,jj,jk,jppoc) / ( trn(ji,jj,jk,jpnum) + rtrn ) / xkr_massp
! -------------- To avoid sinking speed over 50 m/day -------
                  znum  = MIN( xnumm(jk), znum )
                  znum  = MAX( 1.1      , znum )
                  znum3d(ji,jj,jk) = znum
!------------------------------------------------------------
                  zeps  = ( zval1 * znum - 1. )/ ( znum - 1. )
                  zfm   = xkr_frac**( 1. - zeps )
                  zgm   = xkr_frac**( zval1 - zeps )
                  zdiv  = MAX( 1.e-4, ABS( zeps - zval2 ) ) * SIGN( 1., ( zeps - zval2 ) )
                  zdiv1 = zeps - zval3
!!gmoptimisation possible below
                  wsbio3(ji,jj,jk) = xkr_wsbio_min * ( zeps - zval1 ) / zdiv    &
     &                             - xkr_wsbio_max *   zgm * xkr_eta  / zdiv
                  wsbio4(ji,jj,jk) = xkr_wsbio_min *   ( zeps-1. )    / zdiv1   &
     &                             - xkr_wsbio_max *   zfm * xkr_eta  / zdiv1
                  IF( znum == 1.1)   wsbio3(ji,jj,jk) = wsbio4(ji,jj,jk)
               ENDIF
            END DO
         END DO
      END DO

      wscal(:,:,:) = MAX( wsbio3(:,:,:), 50. )


!   INITIALIZE TO ZERO ALL THE SINKING ARRAYS
!   -----------------------------------------

      sinking (:,:,:) = 0.e0
      sinking2(:,:,:) = 0.e0
      sinkcal (:,:,:) = 0.e0
      sinkfer (:,:,:) = 0.e0
      sinksil (:,:,:) = 0.e0

!   Compute the sedimentation term using p4zsink2 for all
!   the sinking particles
!   -----------------------------------------------------

      CALL p4z_sink2( wsbio3, sinking , jppoc )
      CALL p4z_sink2( wsbio4, sinking2, jpnum )
      CALL p4z_sink2( wsbio3, sinkfer , jpsfe )
      CALL p4z_sink2( wscal , sinksil , jpdsi )
      CALL p4z_sink2( wscal , sinkcal , jpcal )

!  Exchange between organic matter compartments due to
!  coagulation/disaggregation
!  ---------------------------------------------------

      zval1 = 1. + xkr_zeta
      zval2 = 1. + xkr_eta
      zval3 = 3. + xkr_eta
      zval4 = 4. + xkr_eta

      DO jk = 1,jpkm1
         DO jj = 1,jpj
            DO ji = 1,jpi
               IF( tmask(ji,jj,jk) /= 0.e0 ) THEN

                  znum = trn(ji,jj,jk,jppoc)/(trn(ji,jj,jk,jpnum)+rtrn) / xkr_massp
! -------------- To avoid sinking speed over 50 m/day -------
                  znum  = min(xnumm(jk),znum)
                  znum  = MAX( 1.1,znum)
!------------------------------------------------------------
                  zeps  = ( zval1 * znum - 1.) / ( znum - 1.)
                  zdiv  = MAX( 1.e-4, ABS( zeps - zval3) ) * SIGN( 1., zeps - zval3 )
                  zdiv1 = MAX( 1.e-4, ABS( zeps - 4.   ) ) * SIGN( 1., zeps - 4.    )
                  zdiv2 = zeps - 2.
                  zdiv3 = zeps - 3.
                  zdiv4 = zeps - zval2
                  zdiv5 = 2.* zeps - zval4
                  zfm   = xkr_frac**( 1.- zeps )
                  zsm   = xkr_frac**xkr_eta

!    Part I : Coagulation dependant on turbulence
!    ----------------------------------------------

                  zagg1 = ( 0.163 * trn(ji,jj,jk,jpnum)**2               &
                     &            * 2.*( (zfm-1.)*(zfm*xkr_mass_max**3-xkr_mass_min**3)    &
                     &            * (zeps-1)/zdiv1 + 3.*(zfm*xkr_mass_max-xkr_mass_min)    &
                     &            * (zfm*xkr_mass_max**2-xkr_mass_min**2)                  &
                     &            * (zeps-1.)**2/(zdiv2*zdiv3))            &
# if defined key_off_degrad
                     &                 *facvol(ji,jj,jk)       &
# endif
                     &    )

                  zagg2 = (  2*0.163*trn(ji,jj,jk,jpnum)**2*zfm*                       &
                     &                   ((xkr_mass_max**3+3.*(xkr_mass_max**2          &
                     &                    *xkr_mass_min*(zeps-1.)/zdiv2                 &
                     &                    +xkr_mass_max*xkr_mass_min**2*(zeps-1.)/zdiv3)    &
                     &                    +xkr_mass_min**3*(zeps-1)/zdiv1)                  &
                     &                    -zfm*xkr_mass_max**3*(1.+3.*((zeps-1.)/           &
                     &                    (zeps-2.)+(zeps-1.)/zdiv3)+(zeps-1.)/zdiv1))      &
#    if defined key_off_degrad
                     &                 *facvol(ji,jj,jk)             &
#    endif
                     &    )

                  zagg3 = (  0.163*trn(ji,jj,jk,jpnum)**2*zfm**2*8. * xkr_mass_max**3   &
#    if defined key_off_degrad
                     &                 *facvol(ji,jj,jk)             &
#    endif
                     &    )

                  zaggsh = ( zagg1 + zagg2 + zagg3 ) * rfact2 * xdiss(ji,jj,jk) / 1000.

!    Aggregation of small into large particles
!    Part II : Differential settling
!    ----------------------------------------------

                  zagg4 = (  2.*3.141*0.125*trn(ji,jj,jk,jpnum)**2*                       &
                     &                 xkr_wsbio_min*(zeps-1.)**2                         &
                     &                 *(xkr_mass_min**2*((1.-zsm*zfm)/(zdiv3*zdiv4)      &
                     &                 -(1.-zfm)/(zdiv*(zeps-1.)))-                       &
                     &                 ((zfm*zfm*xkr_mass_max**2*zsm-xkr_mass_min**2)     &
                     &                 *xkr_eta)/(zdiv*zdiv3*zdiv5) )                     &
# if defined key_off_degrad
                     &                 *facvol(ji,jj,jk)        &
# endif
                     &    )

                  zagg5 = (  2.*3.141*0.125*trn(ji,jj,jk,jpnum)**2                         &
                     &                 *(zeps-1.)*zfm*xkr_wsbio_min                        &
                     &                 *(zsm*(xkr_mass_min**2-zfm*xkr_mass_max**2)         &
                     &                 /zdiv3-(xkr_mass_min**2-zfm*zsm*xkr_mass_max**2)    &
                     &                 /zdiv)                   &
# if defined key_off_degrad
                     &                 *facvol(ji,jj,jk)        &
# endif
                     &    )

                  zaggsi = ( zagg4 + zagg5 ) * zstep / 10.

                  zagg = 0.5 * xkr_stick * ( zaggsh + zaggsi )

!     Aggregation of DOC to small particles
!     --------------------------------------

                  zaggdoc = ( 0.4 * trn(ji,jj,jk,jpdoc)               &
                     &        + 1018.  * trn(ji,jj,jk,jppoc)  ) * zstep    &
# if defined key_off_degrad
                     &        * facvol(ji,jj,jk)                              &
# endif
                     &        * xdiss(ji,jj,jk) * trn(ji,jj,jk,jpdoc)
                      
                  znumdoc = trn(ji,jj,jk,jpnum) / ( trn(ji,jj,jk,jppoc) + rtrn )
                  tra(ji,jj,jk,jppoc) = tra(ji,jj,jk,jppoc) + zaggdoc
                  tra(ji,jj,jk,jpnum) = tra(ji,jj,jk,jpnum) + zaggdoc * znumdoc - zagg
                  tra(ji,jj,jk,jpdoc) = tra(ji,jj,jk,jpdoc) - zaggdoc

               ENDIF
            END DO
         END DO
      END DO

#    if defined key_trc_dia3d
      zrfact2 = 1.e3 * rfact2r
      trc2d(:,:, 5)   = sinking (:,:,iksed+1) * zrfact2
      trc2d(:,:, 6)   = sinking2(:,:,iksed+1) * zrfact2
      trc2d(:,:, 7)   = sinkfer (:,:,iksed+1) * zrfact2
      trc2d(:,:, 9)   = sinksil (:,:,iksed+1) * zrfact2
      trc2d(:,:,10)   = sinkcal (:,:,iksed+1) * zrfact2
      trc3d(:,:,:,12) = sinking (:,:,:)       * zrfact2
      trc3d(:,:,:,13) = sinking2(:,:,:)       * zrfact2
      trc3d(:,:,:,14) = sinksil (:,:,:)       * zrfact2
      trc3d(:,:,:,15) = sinkcal (:,:,:)       * zrfact2
      trc3d(:,:,:,16) = znum3d  (:,:,:)
      trc3d(:,:,:,17) = wsbio3  (:,:,:)
      trc3d(:,:,:,18) = wsbio4  (:,:,:)
#    endif
      !
       IF(ln_ctl)   THEN  ! print mean trends (used for debugging)
         WRITE(charout, FMT="('sink')")
         CALL prt_ctl_trc_info(charout)
         CALL prt_ctl_trc(tab4d=tra, mask=tmask, clinfo=ctrcnm)
       ENDIF
   END SUBROUTINE p4z_sink_kriest

#else
   !!======================================================================
   !!  Dummy module :                                   No PISCES bio-model
   !!======================================================================
CONTAINS
   SUBROUTINE p4z_sink_kriest                    ! Empty routine
   END SUBROUTINE p4z_sink_kriest
#endif 

   !!======================================================================
END MODULE  p4zsink_kriest
